//+------------------------------------------------------------------+
//|                                                      StarINT.mq5 |
//|                                  Copyright 2021, MetaQuotes Ltd. |
//|                         https://www.mql5.com/ru/users/nikolay7ko |
//+------------------------------------------------------------------+
#property copyright "Copyright 2021, Nikolay Semko"
#property link      "https://www.mql5.com/ru/users/nikolay7ko"
#property link      "SemkoNV@bk.ru"
#property version   "1.00"
#property indicator_chart_window
#include <Canvas\iCanvas_CB.mqh> //https://www.mql5.com/ru/code/22164

#property indicator_chart_window
#property indicator_plots   1
#property indicator_buffers 1
  
input int NS = 10000; // Total stars
input bool AutoRotation = true; 
input double RotationSpeed = 1.1;
input double MaxSizeRadius = 0.4;

struct Star
  {
   double             x;
   double             y;
   double             z;
   double             size;
  };
Star st[];
double X0,Y0;
double _a,_b,_x,_y,z2,_r;
double per=1;
double K;
//+------------------------------------------------------------------+
int OnInit()
  {
   ChartSetInteger(0,CHART_FOREGROUND,true);
   ArrayResize(st,NS);
   //ChartSetInteger(0,CHART_SHOW,false);
   StarArray(st);
   _a =1.0;
   _b = 1.0;
   per=20*_Height;
   K=100.0*_Height;
   _r = _Height*1.5;
   X0=_Width/2.0;
   Y0=_Height/2.0;

   Draw();
   if (AutoRotation) EventSetMillisecondTimer(20);
   return(INIT_SUCCEEDED);
  }
//+------------------------------------------------------------------+
void OnTimer() {
   _a += 0.0001*RotationSpeed;
   _b += 0.00007*RotationSpeed; 
   Draw();
}
//+------------------------------------------------------------------+
void OnDeinit(const int reason)
  {
  // if(reason<2)
  // ChartSetInteger(0,CHART_SHOW,true);
  }
//+------------------------------------------------------------------+
int OnCalculate(const int rates_total,const int prev_calculated,const int begin,const double &price[])
  {
   Draw();
   return(rates_total);
  }
//+------------------------------------------------------------------+
void OnChartEvent(const int id,
                  const long &lparam,
                  const double &dparam,
                  const string &sparam)
  {
   if(id == CHARTEVENT_CHART_CHANGE) Draw();
   if(id == CHARTEVENT_MOUSE_MOVE) {
      _a=2.0*(1.1*_Width  - _MouseX)/_Width;
      _b=1.0*(1.1*_Height - _MouseY)/_Height;
      Draw();
   }
  }
//+------------------------------------------------------------------+
void Draw()
  {
   //ulong t = GetMicrosecondCount();
   Canvas.Erase(0xFF000000);
   for(int i = 0; i<NS; i++)
     {
      _3D(st[i].x*_Width,st[i].y*_Height,st[i].z*_Height);
      Dot(_x,_y,st[i].size*(1.0+0.2*(z2/(1.5*_Height))),0x00FFFFFF);
     }
   Canvas.Update();
   //t = GetMicrosecondCount() - t;
   //Comment(t);
  }
//+------------------------------------------------------------------+
void _3D(double x, double y, double z)
  {
   if(_a <0 || _b<0 || _a>=per || _b>=per)
      return;
   double x1=x*cos(_a)+z*sin(_a);
   double z1=-x*sin(_a)+z*cos(_a);
   double y1=y*cos(_b)+z1*sin(_b);
   z2=-y*sin(_b)+z1*cos(_b);
   z2+=_r;
   _x=(X0+K*x1/(z2+K));
   _y=(Y0+K*y1/(z2+K));
  }
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
void StarArray(Star &arr[])
  {
   for(int i = 0; i<NS; i++)
     {
      arr[i].x = 3*rand()/32767.0 - 1.5;
      arr[i].y = 3*rand()/32767.0 - 1.5;
      arr[i].z = 3*rand()/32767.0 - 1.5;
      arr[i].size = MaxSizeRadius*rand()/32767.0;
     }
  }
//+------------------------------------------------------------------+
void Dot(double x,double y,double r, uint clr)
  {
   int w=W.Width;
   uchar tr = (r<0.5)?(uchar(255*r*r/0.25)):255;
   Canvas.FillCircle(Round(x),Round(y), Round(r), ColorToARGB(clr,tr));
  }
